Module: Fluent::GoogleCloudOutput::CredentialsInfo

Defined in:
lib/fluent/plugin/out_google_cloud.rb

Overview

TODO: This functionality should eventually be available in another library, but implement it ourselves for now.

Class Method Summary collapse

Class Method Details

.extract_project_id(str) ⇒ Object

Extracts the project id (either name or number) from str and returns it (as a string) on success, or nil on failure.

Recognizes IAM format ([email protected]) as well as the legacy format with a project number at the front of the string, terminated by a dash (-) which is not part of the ID, i.e.: 270694816269-1l1r2hb813leuppurdeik0apglbs80sv.apps.googleusercontent.com



1241
1242
1243
1244
1245
1246
1247
1248
# File 'lib/fluent/plugin/out_google_cloud.rb', line 1241

def self.extract_project_id(str)
  [/^.*@(?<project_id>.+)\.iam\.gserviceaccount\.com/,
   /^(?<project_id>\d+)-/].each do |exp|
    match_data = exp.match(str)
    return match_data['project_id'] unless match_data.nil?
  end
  nil
end

.project_idObject

Determine the project ID from the credentials, if possible. Returns the project ID (as a string) on success, or nil on failure.



1221
1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
# File 'lib/fluent/plugin/out_google_cloud.rb', line 1221

def self.project_id
  creds = Google::Auth.get_application_default(LOGGING_SCOPE)
  if creds.issuer
    id = extract_project_id(creds.issuer)
    return id unless id.nil?
  end
  if creds.client_id
    id = extract_project_id(creds.client_id)
    return id unless id.nil?
  end
  nil
end